--
-- ChangeLowerLink
--
-- @package 	fs17.saphir_plowstar.specializations.changeLowerLink
-- @auhtor 		IceBlade <www.cg-production.com>
-- @copyright 	2018 CG-Production
-- @license    	www.cg-production.com/license/FS_Script_Licence_2018_1.txt
-- @version		1.0.0.1
-- @history		<1.0.0.0> <04.03.2018> creation
-- 				<1.0.0.1> <05.03.2018> bugfix for attacher
-- 

ChangeLowerLink = {};

function ChangeLowerLink.prerequisitesPresent(specializations)
	return true;
end;

function ChangeLowerLink:preLoad()
	self.changePosition = ChangeLowerLink.changePosition;
	self.setPosition = ChangeLowerLink.setPosition;
end;

function ChangeLowerLink:load()
	local baseKey = 'vehicle.positions';

	self.updatedAttacher = getXMLInt(self.xmlFile, baseKey .. '#updatedAttacher');

	self.positions = {};

	local i = 0;
	while true do
		local key = string.format('%s.position(%d)', baseKey, i);
		if not hasXMLProperty(self.xmlFile, key) then
			break;
		end;
		local id = getXMLInt(self.xmlFile, key .. '#id');

		if self.positions[id] == nil then
			local position = {};
			position.id = id;
			position.elements = {};

			local s = 0;
			while true do
				local elementsKey = string.format('%s.element(%d)', key, s);
				if not hasXMLProperty(self.xmlFile, elementsKey) then
					break;
				end;
				local element = {};
				element.index = Utils.indexToObject(self.components, getXMLString(self.xmlFile, elementsKey .. '#index'));
				
				local rotation = Utils.getNoNil(getXMLString(self.xmlFile, elementsKey .. '#rotation'), false);
				if rotation ~= false then
					local x,y,z = Utils.getVectorFromString(rotation);
					element.rotation = {};
					element.rotation.x = x;
					element.rotation.y = y;
					element.rotation.z = z;
				else
					element.rotation = false;
				end;

				local translation = Utils.getNoNil(getXMLString(self.xmlFile, elementsKey .. '#translation'), false);
				if translation ~= false then
					local x,y,z = Utils.getVectorFromString(translation);
					element.translation = {};
					element.translation.x = x;
					element.translation.y = y;
					element.translation.z = z;
				else
					element.translation = false;
				end;

				table.insert(position.elements, element);
				s = s + 1;
			end;

			self.positions[id] = position;
		else
			error('[ChangeLowerLink.lua] Position ID ' .. id .. ' is used multiple.');
			break;
		end;

		i = i + 1;
	end;


	self.currentPosition = 0;
	self:setPosition(self.currentPosition, true);
end;

function ChangeLowerLink:postLoad(savegame)
	if savegame ~= nil and not savegame.resetVehicles then
		local currentPosition = getXMLInt(savegame.xmlFile, savegame.key .. '#currentLowerLinkPosition');
		if currentPosition ~= nil then
			self.currentPosition = currentPosition;
		end;
	end;

	self:setPosition(self.currentPosition, true);
end;

function ChangeLowerLink:getSaveAttributesAndNodes(nodeIdent)
	local currentPosition = ('currentLowerLinkPosition="%s"'):format(self.currentPosition);
	return currentPosition;
end;

function ChangeLowerLink:delete()
end;

function ChangeLowerLink:readStream(streamId, connection)
end;

function ChangeLowerLink:writeStream(streamId, connection)
end;

function ChangeLowerLink:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ChangeLowerLink:keyEvent(unicode, sym, modifier, isDown)
end;

function ChangeLowerLink:update(dt)
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA4) then
			-- lowerlink down
			if self.positions[self.currentPosition - 1] ~= nil then
				self:changePosition('down', false);
			end;
		end;
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
			-- lowerlink up
			if self.positions[self.currentPosition + 1] ~= nil then
				self:changePosition('up', false);
			end;
		end;
	end;
end;

function ChangeLowerLink:updateTick(dt)
end;

function ChangeLowerLink:draw()
	if self.attacherJoints[self.updatedAttacher].jointIndex == 0 then
		if self.positions[self.currentPosition + 1] ~= nil then
			g_currentMission:addHelpButtonText(g_i18n:getText("function_lowerLinkUp"), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);
		end;
		if self.positions[self.currentPosition - 1] ~= nil then
			g_currentMission:addHelpButtonText(g_i18n:getText("function_lowerLinkDown"), InputBinding.IMPLEMENT_EXTRA4, nil, GS_PRIO_HIGH);
		end;
	end;
end;

function ChangeLowerLink:changePosition(currentDirection, noEventSend)
	if self.currentPosition ~= nil then
		if self.attacherJoints[self.updatedAttacher].jointIndex == 0 then
			if currentDirection ~= nil then
				if currentDirection == 'up' then
					if self.positions[self.currentPosition + 1] ~= nil then
						self.currentPosition = self.currentPosition + 1;
					end;
				end;
				if currentDirection == 'down' then
					if self.positions[self.currentPosition - 1] ~= nil then
						self.currentPosition = self.currentPosition - 1;
					end;
				end;
			end;
			if self.positions[self.currentPosition] ~= nil then
				self:setPosition(self.currentPosition, noEventSend);
			end;
		end;
	end;
end;

function ChangeLowerLink:setPosition(currentPosition, noEventSend)
	if currentPosition ~= nil then
		if self.positions[currentPosition] ~= nil then
			for e, element in pairs(self.positions[currentPosition].elements) do
				if element.rotation ~= false then
					setRotation(element.index, math.rad(element.rotation.x), math.rad(element.rotation.y), math.rad(element.rotation.z));
				end;
				if element.translation ~= false then
					setTranslation(element.index, element.translation.x, element.translation.y, element.translation.z);
				end;
			end;
			self.currentPosition = currentPosition;
			ChangeLowerLinkEvents.sendEvent(self, currentPosition, noEventSend);
		end;
	end;
end;